﻿--[[ blank-module.lua
     ----------------
 
This file is meant to be a skeleton containing the most basic elements
required for all Prat 2 modules which can be used as a template.

The following strings are used through the file and should be replaced
with the relevant information:

   Signon = ModuleName 	-- eg, ExampleModule or AddonMsgs or Whatever
   ## = Description	-- eg, "An blank example module file for Prat 2.0."
   %% = revision number	-- eg, 22222

Other variables that need to be set have been marked by the following
on a line by itself:

-- !!

For more resources on Prat, its modules, and Acey type stuff, have a
look at some of this page:

 - http://groups.google.com/group/wow-prat/web/prat-resources
 
Cheers!

 - Fin // fin@instinct.org
]]

-- !!
--[[
Name: Prat_Signon
Revision: $Revision: 01$
-- !!
Your Name (youremail@example.com)
Author(s):  (Chiper)
Website: http://files.wowace.com/Prat/
Documentation: To be written
Subversion: n/a
Discussions: n/a
Issues and feature requests: #wowace on Freenode IRC
Description: Adds extra player information to guild and friend signon/signoff messages.
Dependencies: Prat
]]

-- Get Utility Libraries
local util, DBG, CLR = GetPratUtils()

-- set prat module name
local PRAT_MODULE = Prat:RequestModuleName("Signon")

-- define localized strings
local L = PRAT_LIBRARY(PRATLIB.LOCALIZATION):new(PRAT_MODULE)

L:RegisterTranslations("enUS", function() return {
	["Signon"] = true,
	["Adds extra player information to guild and friend signon/signoff messages"] = true,

	["Guild Signon String"] = true,
	["Format string for when guild members sign on"] = true,
	["Guild Signoff String"] = true,
	["Format string for when guild members log off"] = true,
	["Friend Signon String"] = true,
	["Format string for when friends sign on"] = true,
	["Friend Signoff String"] = true,
	["Format string for when friends log off"] = true,
	["&name [&level &class] <Guild> &rank (Note: &note) has come online in &zone"] = true,
	["&name [&level &class] <Guild> &rank (Note: &note) has logged off"],
	["&name [&level &class] <Friend> has signed on in &zone"],
	["&name [&level &class] <Friend> has signed off"]
	} end)

--[[
	translations for various locales here - typically including
	deDE, frFR, esES, koKR, zhCN, zhTW; must include an entry for each
	text string listed for enUS, eg:
]]

-- get prat module categories
local cat = Prat.Categories

-- create Prat module
Prat_Signon = Prat:NewModule(PRAT_MODULE)
Prat_Signon.revision = tonumber(string.sub("$Revision$", 12, -3))

-- define key module values
Prat_Signon.moduleName	= L["Signon"] 	-- module name, eg "ExampleMod"
-- !!
Prat_Signon.moduleDesc	= L["Adds extra player information to guild and friend signon/signoff messages"] 	-- module description, eg "An example of a Prat 2 mod"

Prat_Signon.consoleName	= string.lower(Prat_Signon.moduleName)
Prat_Signon.guiName		= Prat_Signon.moduleName

-- categories the module is related to, eg "{ cat.TEXT, cat.INFO }"
-- list here: http://groups-beta.google.com/group/wow-prat/web/prat-categories
-- !!
Prat_Signon.Categories	= {cat.FILTER, cat.FORMATTING, cat.INFO}

-- init moduleOoptions (populated by the other various options tables below)
Prat_Signon.moduleOptions = {}

-- default values for any settings that need them
-- !!
Prat_Signon.defaultDB = {
	on	= true, -- this value tracks if the module should be on or off, 
	             -- an option 'toggle' is automatically created so you can 
	             -- set this value
	guild_signon_format = L["&name [&level &class] <Guild> &rank (Note: &note) has come online in &zone"],
	guild_signoff_format = L["&name [&level &class] <Guild> &rank (Note: &note) has logged off"],
	friend_signon_format = L["&name [&level &class] <Friend> has signed on in &zone"],
	friend_signoff_format = L["&name [&level &class] <Friend> has signed off"]
	}


-- any boolean options
Prat_Signon.toggleOptions = {
	--[[
	NB: for each key name, you must have two corresponding local
	entries for the key name and description:

		key_name
		key_desc

	eg, with an entry of:

		cheesy	= true,

	You would then need to add the two entries shown below:

	L:RegisterTranslations("enUS", function() return {
	...
		["cheesy_name"] = "Cheesy",
		["cheesy_desc"] = "Whether the addon exudes an aura of cheese, or not.",
	...
	} end)
	]]
	}

function Prat_Signon:GetModuleOptions()
	self.moduleOptions = {
		name	= self.moduleName,
		desc	= self.moduleDesc,
		type	= "group",
		args = {
			--[[
				table of possible arguments to the slash command and for the FuBar menu;
				see here for more details on valid entries:
			
				- http://www.wowace.com/wiki/AceConsole-2.0
				- http://www.wowace.com/wiki/AceOptions_data_table
			]]
			
			--[[ eg:
			
			test = {
				name	= L["Test setting"],				-- used in tooltip and generated help
				desc	= L["To test assigning a value to an option."],	-- used in tooltip and generated help
				type	= "text",					-- type of argument (eg, group, toggle
				order	= 99,						-- order when displaying list of args
				usage	= L["test <value>"],				-- text to to display if no values given

				get	= function() return self.db.profile.test end,	-- function to return the current value
				set	= function(v) self.db.profile.test = v end,	-- function to set this argument's value
				},
			]]
			}
		}
	
	return self.moduleOptions
end

--[[------------------------------------------------
    Module Event Functions
------------------------------------------------]]--

-- things to do when the module is enabled
function Prat_Signon:OnModuleEnable()
	-- register events
	-- !!
	self:RegisterEvent("Prat_PreAddMessage")
end

-- things to do when the module is disabled
function Prat_Signon:OnModuleDisable()
	-- unregister events
	self:UnregisterAllEvents()
end

--[[------------------------------------------------
    Core Functions
------------------------------------------------]]--

function Prat_Signon:GetUserData(playerName)
	-- Search guild
	for i=1, GetNumGuildMembers(true) do
		u = {type = "GUILD", name, rank, rankIndex, level, class, zone, note, officernote, online, status};
		u.name, u.rank, u.rankIndex, u.level, u.class, u.zone, u.note, u.officernote, u.online, u.status = GetGuildRosterInfo(i);
		if (string.find(playerName, u.name)) then
			out = "";
			u.name = "|Hplayer:"..u.name.."|h"..u.name.."|h";
			return u;
		end
	end
	-- Then search friends
	for i=1, GetNumFriends() do
		u = {type = "FRIEND", name, level, class, zone, online, status, note, rank};
		u.name, u.level, u.class, u.zone, u.online, u.status = GetFriendInfo(i);
		if (string.find(playerName, u.name)) then
			out = "";
			u.name = "|Hplayer:"..u.name.."|h"..u.name.."|h";
			return u;
		end
	end
end

function Prat_Signon:Prat_PreAddMessage(message, frame, event, t, r, g, b)
	local pName
	local pOnline
	if event == "CHAT_MSG_SYSTEM" then -- only process system messages
		if (string.find(message.MESSAGE, "online")) then -- user came online
			pName = message.PLAYER
			pOnline = true
		elseif (string.find(message.MESSAGE, "offline")) then -- user went offline
			pName = message.MESSAGE:match("(%a+)")
			pOnline = false
		else
			return
		end
		
		-- have captured player name, and broken out of the function if this isnt the event we want.
		local uData = Prat_Signon:GetUserData(pName)
		if uData == nil then 
			return  -- couldn't get information, don't alter the text.
		end
		
		-- safe values are: name, level, class, zone, note, and rank
		local parseString
		local parseCount
		if uData.type == "GUILD" then
			if pOnline then
				parseString = Prat_Signon.db.profile.guild_signon_format
			else
				parseString = Prat_Signon.db.profile.guild_signoff_format
			end
		else
			if pOnline then
				parseString = Prat_Signon.db.profile.friend_signon_format
			else
				parseString = Prat_Signon.db.profile.friend_signoff_format
			end
		end
		
		parseString, parseCount = string.gsub(parseString, "&name", uData.name)
		parseString, parseCount = string.gsub(parseString, "&level", tostring(uData.level))
		parseString, parseCount = string.gsub(parseString, "&class", uData.class)
		parseString, parseCount = string.gsub(parseString, "&zone", uData.zone)
		parseString, parseCount = string.gsub(parseString, "&rank", uData.rank)
		parseString, parseCount = string.gsub(parseString, "&note", uData.note)
		
		-- nil out all message data except actual content.  We have to do this because the signon name gets parsed.  Oddly, it doesn't happen for signoff.
		message.pP = ""
		message.lL = ""
		message.PLAYERLINK = ""
		message.PLAYERLINKDATA = ""
		message.LL = ""
		message.PLAYER = ""
		message.Ll = ""
		message.Pp = ""
		message.MESSAGE = parseString
		
		
	end
end
